﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace VisualStateManager
{
    public class CompositeCondition : ICondition
    {
        // Private fields ------------------------------------------------------

        private CompositionKind compositionKind;
        private List<ICondition> conditions;
        private bool value;

        // Private methods -----------------------------------------------------

        private void HandleValueChanged(object sender, ValueChangedEventArgs e)
        {
            ReevaluateValue();
        }

        private bool InternalEvalValue()
        {
            if (conditions.Count == 0)
                return false;
            else
            {
                switch (compositionKind)
                {
                    case CompositionKind.And:
                        {
                            bool result = true;

                            int i = 0;
                            while (i < conditions.Count && result)
                                result &= conditions[i++].Value;

                            return result;
                        }
                    case CompositionKind.Or:
                        {
                            bool result = false;

                            int i = 0;
                            while (i < conditions.Count && !result)
                                result |= conditions[i++].Value;

                            return result;
                        }
                    default:
                        throw new InvalidEnumArgumentException("Unsupported composition kind!");
                }
            }
        }

        private void ReevaluateValue()
        {
            bool newValue = InternalEvalValue();

            if (newValue != value)
            {
                value = newValue;
                OnValueChanged(value);
            }
        }

        // Protected methods ---------------------------------------------------

        protected void OnValueChanged(bool newValue)
        {
            if (ValueChanged != null)
                ValueChanged(this, new ValueChangedEventArgs(newValue));
        }

        // Public types --------------------------------------------------------

        public enum CompositionKind
        {
            And,
            Or
        }

        // Public methods ------------------------------------------------------

        public CompositeCondition(CompositionKind kind = CompositionKind.And)
        {
            compositionKind = kind;
            conditions = new List<ICondition>();
            value = false;
        }

        public CompositeCondition(CompositionKind kind,
            params ICondition[] compositeConditions)
        {
            compositionKind = kind;
            conditions = new List<ICondition>();
            value = false;

            foreach (ICondition condition in compositeConditions)
            {
                Add(condition);
            }
        }

        public void Add(ICondition condition)
        {
            if (condition == null)
                throw new ArgumentNullException("condition");

            if (!conditions.Contains(condition))
            {
                condition.ValueChanged += HandleValueChanged;
                conditions.Add(condition);

                ReevaluateValue();
            }
        }

        public void Remove(ICondition condition)
        {
            if (conditions.Contains(condition))
            {
                condition.ValueChanged -= HandleValueChanged;
                conditions.Remove(condition);

                ReevaluateValue();
            }
        }

        // Public properties ---------------------------------------------------

        public bool Value
        {
            get
            {
                return value;
            }
        }

        public event ValueChangedHandler ValueChanged;
    }
}
